using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using Indexing.Windows.Media;

namespace MediaDetectorSample {
    public partial class MainForm : Form {
        public MainForm() {
            InitializeComponent();
        }

        FrameGrabber _fgber;

        delegate void Invoker(object o);

        private void VK쐬NToolStripButton_Click(object sender, EventArgs e) {
            Clear();
        }

        private void JOToolStripButton_Click(object sender, EventArgs e) {
            OpenFile();
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e) {
            if(_fgber != null) {
                _fgber.GrabStarted -= _fgber_GrabStarted;
                _fgber.GrabCompleted -= _fgber_GrabCompleted;
                _fgber.GrabProgressing -= _fgber_GrabProgressing;
                _fgber.Dispose();
            }
        }

        void SetStatusText(object arg) {
            if(InvokeRequired) {
                this.BeginInvoke(new Invoker(SetStatusText), new object[] { arg });
            } else {
                string text = (string)arg;
                toolStripStatusLabel1.Text = text;
            }
        }

        void SetTrackBarValue(object arg) {
            if(InvokeRequired) {
                this.BeginInvoke(new Invoker(SetTrackBarValue), new object[] { arg });
            } else {
                int value = (int)arg;
                trackBar1.Value = value;
            }
        }

        private void button1_Click(object sender, EventArgs e) {
            if(_fgber == null) {
                return;
            }
            // gbNo[̈ʒut[̎擾Jn
            TimeSpan from = TimeSpan.FromSeconds((int)trackBar1.Value);
            TimeSpan to = from + TimeSpan.FromSeconds((int)numericUpDown1.Value);
            _fgber.Detect(from, to);
        }

        private void button2_Click(object sender, EventArgs e) {
            if(_fgber == null) {
                return;
            }
            // ~
            _fgber.Stop();
        }

        private void button3_Click(object sender, EventArgs e) {
            if(_fgber == null) {
                return;
            }
            // ꎞ~
            _fgber.Pause();
        }

        private void button4_Click(object sender, EventArgs e) {
            if(_fgber == null) {
                return;
            }
            // ĊJ
            _fgber.Play();
        }

        private void button5_Click(object sender, EventArgs e) {
            if(_fgber == null) {
                return;
            }
            // ꎞ~(Ot̒~Ȃ)
            _fgber.Suspend();
        }

        private void button6_Click(object sender, EventArgs e) {
            if(_fgber == null) {
                return;
            }
            // ĊJ(SuspendԂ̍ĊJ)
            _fgber.Resume();
        }

        void Clear() {
            if(_fgber == null) {
                return;
            }
            _fgber.Dispose();
            _fgber = null;
            pictureBox1.Image = null;
            toolStripStatusLabel1.Text = "";
            toolStripStatusLabel2.Text = "";
            toolStripStatusLabel2.Image = null;
            toolStripStatusLabel3.Text = "";
        }

        void OpenFile() {
            if(_fgber != null) {
                Clear();
            }
            using(OpenFileDialog dlg = new OpenFileDialog()) {
                dlg.Filter = "fBA t@C(*.wmv;*.mpg;*.avi)|*.wmv;*.mpg;*.avi|ׂẴt@C|*.*";
                if(dlg.ShowDialog() == DialogResult.OK) {
                    try {
                        Cursor = Cursors.WaitCursor;
                        trackBar1.Value = 0;

                        try {
                            // t[Oo
                            _fgber = new FrameGrabber(dlg.FileName);
                        } catch(VideoNotRenderedException ex) {
                            MessageBox.Show(ex.FileName, "fBAFł܂ł", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            trackBar1.Maximum = 0;
                            return;
                        }

                        // Cxgnhw肷
                        _fgber.GrabStarted += _fgber_GrabStarted;
                        _fgber.GrabCompleted += _fgber_GrabCompleted;
                        _fgber.GrabProgressing += _fgber_GrabProgressing;

                        toolStripStatusLabel1.Text = "ready";
                        toolStripStatusLabel2.Text = Path.GetFileNameWithoutExtension(_fgber.FileName);
                        toolStripStatusLabel2.Image = Icon.ExtractAssociatedIcon(_fgber.FileName).ToBitmap();
                        TimeSpan len = _fgber.StreamLength;
                        toolStripStatusLabel3.Text = String.Format("{0:D2}:{1:D2}:{2:D2}", len.Hours, len.Minutes, len.Seconds);

                        // fBA̒gbNo[ɐݒ肷
                        trackBar1.Maximum = (int)_fgber.StreamLength.TotalSeconds;

                        // 擪t[\
                        Bitmap bitmap = new Bitmap(_fgber.GetFrame(TimeSpan.Zero), pictureBox1.Size);
                        DrawTime(bitmap, TimeSpan.Zero, TimeSpan.Zero);
                        pictureBox1.Image = bitmap;
                    } finally {
                        Cursor = Cursors.Default;
                    }
                }
            }
        }

        /// <summary>
        /// Jn
        /// </summary>
        TimeSpan _start;

        void DrawTime(Bitmap bitmap, TimeSpan current, TimeSpan tc) {
            using(Graphics gr = Graphics.FromImage(bitmap)) {
                gr.DrawString(current.ToString(), this.Font, Brushes.Orange, 0, 0);
                if(tc.Ticks > 0) {
                    gr.DrawString(tc.ToString(), this.Font, Brushes.Orange, 0, bitmap.Height - Font.Height);
                }
            }
        }

        /// <summary>
        /// JnCxgnh
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        void _fgber_GrabStarted(FrameGrabber sender, FrameGrabber.GrabStartedEventArgs args) {
            SetStatusText("擾:" + args.Time.ToString());
            _start = args.Time;
        }

        /// <summary>
        /// ICxgnh
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        void _fgber_GrabCompleted(FrameGrabber sender, FrameGrabber.GrabCompletedEventArgs args) {
            SetStatusText("擾:");
        }

        /// <summary>
        /// t[擾Cxgnh
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        void _fgber_GrabProgressing(FrameGrabber sender, FrameGrabber.GrabProgressingEventArgs args) {
            Bitmap bmp = new Bitmap(args.Bitmap, pictureBox1.Size);
            TimeSpan current = _start + args.Time;
            DrawTime(bmp, current, args.Time);
            pictureBox1.Image = bmp;
            int sec = (int)(current.TotalSeconds);
            SetTrackBarValue(sec);
        }

        private void trackBar1_ValueChanged(object sender, EventArgs e) {
            if(_fgber != null) {
                if( !_fgber.IsRunning ) {
                    // t[擾ĕ\
                    TimeSpan pos = TimeSpan.FromSeconds(trackBar1.Value);
                    Bitmap bmp = _fgber.GetFrame(pos);
                    if(bmp != null) {
                        Bitmap thumb = new Bitmap(bmp, pictureBox1.Size);
                        DrawTime(thumb, pos, TimeSpan.Zero);
                        pictureBox1.Image = thumb;
                    }
                }
            }
        }
    }
}